// SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0

#ifndef __ACGNUEX_H__
#define __ACGNUEX_H__

#include <acpi/platform/acgnu.h>

#ifdef __KERNEL__

#define ACPI_USE_LOCAL_CACHE

/*
 * Overrides for in-kernel ACPICA
 */
acpi_status ACPI_INIT_FUNCTION acpi_os_initialize(void);
acpi_status acpi_os_terminate(void);
void acpi_os_printf(const char *format, ...);

static inline void *acpi_os_allocate(acpi_size size)
{
	return (void *)malloc (size);
}

static inline void *acpi_os_allocate_zeroed(acpi_size size)
{
	return (void *)calloc (1, size);
}

static inline void acpi_os_free(void *memory)
{
	free (memory);
}

static inline acpi_thread_id acpi_os_get_thread_id(void)
{
	return 1;
}

static inline u8 acpi_os_readable(void *pointer, acpi_size length)
{
	return TRUE;
}

static inline acpi_status acpi_os_initialize_debugger(void)
{
	return 0;
}

static inline void acpi_os_terminate_debugger(void)
{
	return;
}

#include <stdio.h>
#include <sys/io.h>
#include <sys/time.h>
#include <stddef.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdarg.h>
#include <limits.h>
#include <hurd.h>

#define ACPI_MAX_TABLES 128

static inline acpi_status
acpi_os_enter_sleep(u8 sleep_state, u32 pm1a, u32 pm1b)
{
  return 0;
}

static inline acpi_status
acpi_os_create_lock(acpi_spinlock *lockp)
{
  return 0;
}

static inline acpi_cpu_flags
acpi_os_acquire_lock(acpi_spinlock lockp)
{
  return 1;
}

static inline void
acpi_os_release_lock(acpi_spinlock lockp, acpi_cpu_flags flags)
{
}

static inline void
acpi_os_delete_lock(acpi_spinlock lockp)
{
}

static inline acpi_status
acpi_os_create_semaphore(u32 max_units, u32 initial_units, acpi_semaphore * handle)
{
  return 0;
}

static inline acpi_status
acpi_os_delete_semaphore(acpi_semaphore handle)
{
  return 0;
}

static inline acpi_status
acpi_os_wait_semaphore(acpi_semaphore handle, u32 units, u16 timeout)
{
  return 0;
}

static inline acpi_status
acpi_os_signal_semaphore(acpi_semaphore handle, u32 units)
{
  return 0;
}

/* Supposed to be 64 bit free-running
 * monotonic timer with 100ns granularity
 */
static inline u64
acpi_os_get_timer(void)
{
  struct timespec now;

  clock_gettime(CLOCK_MONOTONIC, &now);
  return (u64)((u64)now.tv_sec * 10000000 + (now.tv_nsec / 100));
}

static inline acpi_status
acpi_os_read_iomem(void *virt_addr, u64 *value, u32 width)
{
  switch (width)
    {
      case 8:
        *(u8 *) value = *((volatile u8 *)virt_addr);
        break;
      case 16:
        *(u16 *) value = *((volatile u16 *)virt_addr);
        break;
      case 32:
        *(u32 *) value = *((volatile u32 *)virt_addr);
        break;
      case 64:
        *(u64 *) value = *((volatile u64 *)virt_addr);
        break;
      default:
        return 1;
    }
  return 0;
}

static inline void
acpi_os_sleep(u64 ms)
{
  usleep(1000 * ms);
}

static inline void
acpi_os_stall(u32 us)
{
  usleep(us);
}

static inline void
acpi_os_wait_events_complete(void)
{
  return;
}

static inline acpi_status
acpi_os_read_port(acpi_io_address port, u32 *value, u32 width)
{
  if (!value)
    return 1;

  switch (width)
    {
      case 8:
        *((u8 *)value) = inb(port);
        return 0;
      case 16:
        *((u16 *)value) = inw(port);
        return 0;
      case 32:
        *((u32 *)value) = inl(port);
        return 0;
      default:
        acpi_os_printf("ACPI: read port invalid width\n");
        return 1;
    }
  return 1;
}

static inline acpi_status
acpi_os_write_port(acpi_io_address port, u32 value, u32 width)
{
  switch (width)
    {
      case 8:
        outb(value, port);
        return 0;
      case 16:
        outw(value, port);
        return 0;
      case 32:
        outl(value, port);
        return 0;
      default:
        acpi_os_printf("ACPI: Bad write port width\n");
        return 1;
    }

  return 1;
}

static inline acpi_status
acpi_os_install_interrupt_handler(u32 gsi, acpi_osd_handler handler,
          void *context)
{
  acpi_os_printf("TODO: add interrupt handler for acpi\n");
  return 0;
}

static inline acpi_status
acpi_os_remove_interrupt_handler(u32 gsi, acpi_osd_handler handler)
{
  acpi_os_printf("TODO: remove interrupt handler for acpi\n");
  return 0;
}

static inline acpi_status
acpi_os_read_pci_configuration(struct acpi_pci_id * pci_id, u32 reg,
             u64 *value, u32 width)
{
  acpi_os_printf("ACPI: Tried to read pci config\n");
  return 1;
}

static inline acpi_status
acpi_os_write_pci_configuration(struct acpi_pci_id * pci_id, u32 reg,
        u64 value, u32 width)
{
  acpi_os_printf("ACPI: Tried to write pci config\n");
  return 1;
}

#endif				/* __KERNEL__ */

#endif				/* __ACGNUEX_H__ */
